<?php
// $Id: xc_search_ajax.inc,v 1.1.2.11 2010/09/16 12:00:24 pkiraly Exp $

function xc_search_ajax_bookmark_item() {
  global $user;
  
  list($nid, $sid) = explode('|', $_GET['id']);
  $success = xc_account_bookmark_item($user->uid, $nid, $sid);
  
  drupal_set_header('Content-Type: text/plain; charset: utf-8');
  printf('{"success": %d}', (int)$success);
  return;
}

function xc_search_ajax_image_info() {
  $isbn = $_GET['ISBN'];
  $upc  = $_GET['UPC'];
  $oclc = $_GET['OCLC'];
  $size = (isset($_GET['large']) && $_GET['large'] == 1) 
    ? SYNDETICS_LARGE 
    : SYNDETICS_SMALL;
  $url = xc_search_get_syndetics_image($isbn, $upc, $oclc, $size);
  drupal_set_header('Content-Type: text/plain; charset: utf-8');
  printf('{"image_url": "%s"}', $url);
}

/**
 * Gets the WorldCat Identity URL, and redirects
 */
function xc_search_ajax_open_worldcat_identity_url() {
  // alternative way, using OpenURL:
  // http://worldcat.org/identities/find
  // ?url_ver=Z39.88-2004
  // &rft_val_fmt=info:ofi/fmt:kev:mtx:identity
  // &rft.namelast=Hickey
  // &rft.namefirst=Thomas
  // &rft_id=info:oclcnum/9746645
  $sURL = 'http://worldcat.org/identities/find?fullName=' . urlencode($_GET['sURL']);
  $content = file_get_contents($sURL);
  $xml = new SimpleXMLElement($content);
  $types = array('ExactMatches', 'FuzzyFirstName', 'FuzzyFamilyName');
  foreach($types as $type) {
    $results = $xml->xpath('/nameAuthorities/match[@type="' . $type .'"]/uri');
    if(!empty($results)) {
      // an example URI:
      // name with LCCN: '/identities/lccn-n79-6533'
      // name without LCCN: '/identities/np-levan, ralph r'
      $uri = $results[0];
      break;
    }
  }
  $url = !empty($uri) ? 'http://worldcat.org' . $uri : $_GET['sURL'];
  drupal_set_header('Location: ' . $url);
  return;
}

function xc_search_ajax_ncip_info() {
  global $_xc_search_ncip_servers;
  
  $ncip_id  = $_GET['ncip_id'];
  $bib_id   = $_GET['bib_id'];
  $availability = '';
  $response = xc_ils_get_availability($ncip_id, array('bib' => array($bib_id)));
  if (isset($response['error'])) {
    $availability = t('Availability Unknown');
    xc_log_error('ncip', $response['error']['ErrorMessage']);
  }
  else {
    // $bib = $status = $response['results']['bib'][$bib_id][0];
    foreach($response['results']['bib'][$bib_id] as $key => $bib) {
      if($bib['CirculationStatus'] == 'Not Charged') {
        $availability = $bib['Location'] . ': <em>' . t('Available') . '</em>';
        break;
      }
    }
  }
  if($availability == '') {
    $availability = '<em>' . t('Not Available') . '</em>';
  }
  xc_log_info('xc', $bib_id . ' => ' . $availability);
  
  if(isset($_GET['debug'])) {
    return $availability;
  }
  
  drupal_json(array(
    'status' => 0, 
    'availability' => $availability
  ));
}

function xc_search_ajax_ncip_info_full() {
  $ncip_item_ids = array();
  $provider_id = $_GET['provider_id'];
  if($_GET['bib_ids'] == '') {
    $locations = '';
  }
  else {
    $bib_ids = explode(',', $_GET['bib_ids']);
    $token       = $_GET['token'];
    xc_ajax_bootstrap($token);
    $ncip_item_ids[$provider_id] = $bib_ids;
    $locations = xc_search_get_full_ncip($ncip_item_ids);
    // theme('manifestation_full_locations', $locations)
  }
  
  if(isset($_GET['debug'])) {
    return $locations;
  }
  
  drupal_json(array(
    'status' => 0, 
    'content' => $locations
  ));
}

function xc_search_get_full_ncip($ncip_ids) {
  global $_xc_search_ncip_servers;
  $rows = array();
  // TODO: put this value into UI's properties
  $max_rows_to_display = 5;

  $has_holdings_info = FALSE;
  foreach ($ncip_ids as $ncip_provider_id => $ncip_item_ids) {
    if(!is_array($ncip_item_ids)) {
      $ncip_item_ids = array($ncip_item_ids);
    }
    $response = xc_ils_get_availability($ncip_provider_id, array('bib' => $ncip_item_ids));
    if (isset($response['error'])) {
      $rows[] = array($_xc_search_ncip_servers[$ncip_provider_id], 
        t('Availability Unknown'), '');
      $msg = t('Error occured while requesting the circulation status of %item: 
        %msg',
        array(
          '%item' => $ncip_item_ids,
          '%msg' => $response['error']['ErrorMessage']
        ));
      xc_log_error('ncip', $response['error']['ErrorMessage']);
    }
    else {
      $i = 0;
      foreach($ncip_item_ids as $ncip_item_id) {
        if(!isset($response['results']['bib'][$ncip_item_id])) {
          $rows[] = array('data' => array(
            $_xc_search_ncip_servers[$ncip_provider_id],  
            t('Availability Unknown'), 
            '', 
            ''
          ));
        }
        else {
          foreach($response['results']['bib'][$ncip_item_id] as $item) {
            // dsm($item);
            $location = (!empty($item['Location'])) 
              ? $item['Location'] 
              : $_xc_search_ncip_servers[$ncip_provider_id];

            $status = (!empty($item['CirculationStatus'])) 
              ? $item['CirculationStatus'] 
              : t('Availability Unknown');
            if ($status == 'Not Charged') {
              $status = t('Available');
            }

            // TODO: admin interface for xc_search_mapit_file, maybe for ILS settings?
            $path = variable_get('xc_search_mapit_file',
              drupal_get_path('module', 'xc_search') . '/demo_stack_directory.pdf');
            $call_number = $item['ItemDescription']['CallNumber'];
            $call_number .= ' > ' . l(t('Map it'), $path, 
              array('attributes' => array('target' => '_blank')));
            $holdings_info = $item['ItemDescription']['HoldingsInformation'];
            if($has_holdings_info == FALSE && !empty($holdings_info)) {
              $has_holdings_info = TRUE;
            }
            $data = array($location, $status, $call_number);
            if ($has_holdings_info) {
              $data[] = $holdings_info;
            };
            $rows[$item['ItemId']] = array('data' => $data);
          }
          // order by ItemId
          krsort($rows);
          $rows = array_values($rows);
        }
      }
    }
  }
  // modify rows if there may be missing last cell, or the number of rows are 
  // greater than maximum displayable
  $hide_rows = count($rows) > $max_rows_to_display;
  if($has_holdings_info || $hide_rows) {
    for($i=0; $i < count($rows); $i++) {
      if($has_holdings_info && count($rows[$i]['data']) == 3) {
        $rows[$i]['data'][] = '';
      }
      if($i >= $max_rows_to_display) {
        $rows[$i]['class'] = 'hidden hidden-1';
      }
    }
    // Add a row, which contains a 'More' link to show all availability info 
    if($hide_rows) {
      $data = array(array(
        'data' => '<a href="javascript:void(0)" ' 
                . 'onclick="XCSearch.showAllLocations(1)">' 
                . t('Show more') . '</a>',
        'align' => 'right',
        'colspan' => ($has_holdings_info ? 4 : 3),
      ));
      $rows[] = array(
        'data' => $data,
        'id' => 'trigger-1'
      );
    }
  }
  
  $header = array(t('Location'), t('Availability'), t('Call number'));
  if($has_holdings_info) { 
    $header[] = t('Volume information');
  }
  return theme('table', $header, $rows);
}

