<?php 
// $Id: xc_search.pages.inc,v 1.1.2.13 2010/09/17 19:47:54 pkiraly Exp $

/**
 * @file
 * This whole file is the implementation of a new search workflow instead of
 * Drupal's default search workflow. The reason to do that is, that those is
 * not fit to XC expectations, and now we can freely modify the original 
 * structure without applying patches or other workaround.
 * The initial state of the process is the modified search module.
 */


function xc_search_view($type = 'xc_search') {
  // Search form submits with POST but redirects to GET. This way we can keep
  // the search query URL clean as a whistle:
  // search/type/keyword+keyword
  if (!isset($_POST['form_id'])) {
    if ($type == '') {
      // Note: search/node can not be a default tab because it would take on the
      // path of its parent (search). It would prevent remembering keywords when
      // switching tabs. This is why we drupal_goto to it from the parent instead.
      drupal_goto('search/node');
    }

    $keys = search_get_keys();
    // Only perform search if there is non-whitespace search term:
    $results = '';
    //if (trim($keys) == '') {
    //  $keys = '*:*';
    //}
    if (trim($keys)) {
      // Log the search keys:
      watchdog('search', '%keys (@type).', array(
        '%keys' => $keys, 
        '@type' => module_invoke($type, 'search', 'name')), 
        WATCHDOG_NOTICE, 
        l(t('results'), 'search/' . $type . '/' . $keys)
      );

      // Collect the search results:
      // we use the Drupal search hook here, to keep as much compatibility as we can
      // it simply provides a wrapper for hook_search and hook_search_page
      $meta = new stdClass();
      $results = xc_search_data($keys, $type, $meta);

      if ($results) {
        $results = theme('box', NULL, $results);
      }
      else {
        $results = theme('box', t('Your search yielded no results'), 
          search_help('search#noresults', drupal_help_arg()));
      }
    }

    $output = '';
    /*
    if(!empty($_GET['ql']) && $_GET['ql'] == 1) {
      $output .= '<div id="xc-search-query">' 
        . t('The query is %query', 
            array('%query' => xc_search_display_query($keys)))
        . '</div>';
    }
    */
    // Construct the search form.
    $output .= drupal_get_form('xc_search_form', NULL, $keys, $type);
    $output .= $results;

    return $output;
  }

  return drupal_get_form('xc_search_form', NULL, empty($keys) ? '' : $keys, $type);
}


/**
 * Perform a standard search on the given keys, and return the formatted results.
 * This function is a modified version of search.module's search_data()
 */
function xc_search_data($keys = NULL, $type = 'xc_search', &$meta = NULL) {

  if (isset($keys)) {
    if (module_hook($type, 'search')) {
      $results = module_invoke($type, 'search', 'search', $keys, $meta);
      if (isset($results) && is_array($results) && count($results)) {
        if (module_hook($type, 'search_page')) {
          return module_invoke($type, 'search_page', $results, $meta);
        }
        else {
          return theme('search_results', $results, $type);
        }
      }
    }
  }
}

function xc_search_form(&$form_state, $action = '', $keys = '', $type = NULL, $prompt = NULL) {
  $rows = intval($_GET['rows']);
  $sort = $_GET['sort'];
  $is_query_link = (!empty($_GET['ql']) && $_GET['ql'] == 1) ? TRUE : FALSE;
  $original_state = $_GET['os'];
  if(($keys == '*' || $keys == '') && !isset($original_state) && isset($_GET['filter'])) {
    $key_default_value = '';
  }
  else if (isset($original_state['keys'])) {
    $key_default_value = $original_state['keys'];
  }
  else {
    $key_default_value = $keys;
  }
  
  // TODO: put this into UI properties
  $sort_options = array(
    '' => t('No sorting'),
    'relevancy' => t('Relevancy'),
    'date_desc' => t('Date (newests first)'),
    'date_asc' => t('Date (oldest first)'),
    'title_asc' => t('Title (A-Z)'),
    'title_desc' => t('Title (Z-A)'),
    'author_asc' => t('Author (A-Z)'),
    'author_desc' => t('Author (Z-A)'),
  );
  
  if(!in_array($sort, array_keys($sort_options))) {
    $sort = 'relevancy';
  }
  
  $form = array(
    '#action' => $action,
    '#attributes' => array('class' => 'search-form'),
    '#theme' => 'xc_search_form',
  );
  
  $form['module'] = array(
    '#type' => 'value',
    '#value' => $type
  );
  
  $form['basic'] = array(
    '#type' => 'item',
    '#title' => $prompt
  );

  $form['basic']['keys'] = array(
    '#type' => 'textfield',
    '#default_value' => $key_default_value,
    '#size' => FALSE,
    '#maxlength' => 255,
  );

  $form['basic']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Search'),
  );

  $form['basic']['sort'] = array(
    '#type' => 'select',
    '#title' => t('Sort by'),
    '#options' => $sort_options,
    '#default_value' => $sort,
    '#attributes' => array('onchange' => 'this.form.submit();'),
  );

  $form['basic']['rows'] = array(
    '#type' => 'select',
    '#title' => t('Results per page'),
    '#options' => drupal_map_assoc(array(10, 25, 50, 100)),
    '#default_value' => $rows,
    '#attributes' => array('onchange' => 'this.form.submit();'),
  );

  // processed_keys is used to coordinate keyword passing between other forms
  // that hook into the basic search form.

  $form['basic']['processed_keys'] = array(
    '#type' => 'value', 
    '#value' => array()
  );
  $form['basic']['original_state'] = array(
    '#type' => 'value', 
    '#value' => array()
  );
  
  
  /*
  $form['basic']['is_query_link'] = array(
    '#type' => 'value', 
    '#value' => $is_query_link
  );
  
  if ($is_query_link) {
    $form['basic']['keys']['#disabled'] = TRUE;
    $form['basic']['keys']['#attributes'] = array('style' => 'display: none;');
    $form['basic']['submit']['#value'] = t('Start new search');
    $form['basic']['hidden_keys'] = array(
      '#type' => 'hidden', 
      '#value' => $keys,
    );
  }
  */
  
  return $form;
}

function theme_xc_search_form($form) {
  drupal_add_css(drupal_get_path('module', 'xc_search') . '/xc_search.css');

  foreach(array('keys', 'submit', 'sort', 'rows') as $key) {
    $rendered[$key] = drupal_render($form['basic'][$key]);
  }
  
  return '<div id="xc-search-keys">' . $rendered['keys'] . '</div>'
       . '<div id="xc-search-hits">' . $rendered['submit'] . '</div>'
       . '<div id="xc-search-info">'
       . '<div id="xc-search-sort">' . $rendered['sort'] . '</div>'
       . '<div id="xc-search-rows">' . $rendered['rows'] . '</div>'
       . '</div>'
       . drupal_render($form)
       . '<div style="clear:both"></div>';
}
